<?php
function requests_process( $method, $url, $auth = null, $payload = null ) {
	if ( isset( $payload ) ) {
			$payload = json_encode( $payload );
	}
	try {
		$response = wp_remote_request(
			$url,
			array(
				'headers' => array(
					'Content-Type'  => 'application/json',
					'Authorization' => $auth,
				),
				'body'    => $payload,
				'method'  => $method,
			)
		);
		if ( is_wp_error( $response ) ) {
			throw new Exception( $response->get_error_message(), $response->get_error_code() );
		} else {
			if ( $response['response']['code'] >= 400 ) {
				// $response = json_decode( $response['body'], true );
				throw new Exception( $response['message'], $response['statusCode'] );
			} else {
				return json_decode( $response['body'], true );
			}
		}
	} catch ( Exception $err ) {
		throw $err;
	}
}

add_action( 'wp_ajax_envia_cities', 'getcities' );

function getcities() {
	try {
			$country = isset( $_GET['country'] ) ? sanitize_text_field( $_GET['country'] ) : null;
			$state   = isset( $_GET['state'] ) ? sanitize_text_field( $_GET['state'] ) : null;
		if ( ! is_null( $country ) && ! is_null( $state ) ) {
			$url    = "https://geocodes.envia.com/list/localities/{$country}/{$state}";
			$cities = requests_process( 'GET', $url, null, null );
			header( 'HTTP/1.1 200 OK' );
			echo json_encode( $cities, JSON_PRETTY_PRINT );
		}
	} catch ( Exception $err ) {
		throw $err;
	}
	die();
}

add_action( 'wp_ajax_envia_forms', 'getForm' );
function getForm() {
	try {
			$country = isset( $_GET['country'] ) ? sanitize_text_field( $_GET['country'] ) : null;
		if ( ! is_null( $country ) ) {
			$url    = "https://queries.envia.com/generic-form?country_code={$country}";
			$cities = requests_process( 'GET', $url, null, null );
			header( 'HTTP/1.1 200 OK' );
			echo json_encode( $cities, JSON_PRETTY_PRINT );
		}
	} catch ( Exception $err ) {
		throw $err;
	}
	die();
}
