<?php
/**
 * Plugin Name: Envia Shipping and Fulfillment
 * Plugin URI: https://www.envia.com/integraciones/woocommerce/
 * Version: 4.0.4
 * Description: The Best Shipping Solution for your business. Connect to 150+ couriers worldwide and get over 70% discounts for domestic and international shipments!
 * Author: Tendecys Innovations
 * Author URI: https://tendencys.com
 * Text Domain: Envia Shipping and Fulfillment
 * Woo: 18734001052053:dc29297b72b473f0e1f97e671bb3f370
 */

ob_start();
define( 'MAINDIR', basename( __DIR__ ) );
define( 'MAINFILE', basename( __DIR__ ) . '/envia-shipping-module.php' );
define( 'ENVIA_HOSTNAME', 'https://ship.envia.com/' );
define( 'ENVIA_QUERIES_HOSTNAME', 'https://queries.envia.com' );
define( 'ENVIA_APP_HOSTNAME', 'https://api-clients.envia.com' );
define( 'ENVIA_LANDING', 'https://envia.com' );
define( 'ENVIA_S3', 'https://s3.us-east-2.amazonaws.com/enviapaqueteria' );
if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
	function initEnviaShipping() {
		require 'controllers/formFieldsController.php';
		require 'functions/functions.php';
		require 'functions/shippingLabels.php';
		if ( class_exists( 'WC_Shipping_Method' ) ) {
			class Envia_Shipping_Module extends WC_Shipping_Method {

				public function __construct( $instance_id = 0 ) {
					$this->id                 = 'envia_shipping';
					$this->instance_id        = absint( $instance_id );
					$this->title              = __( 'Live shipping rates', 'Envia.com' );
					$this->method_title       = __( 'Envia rates and shipping', 'Envia.com' );
					$this->method_description = __( 'Send easily WooCommerce orders with Envia and Rate Shipment Cost.', '' );
					$this->supports           = array( 'shipping-zones', 'settings', 'instance-settings', 'instance-settings-modal' );
					$this->enabled            = 'yes';
					$this->origin_options     = get_option( 'woocommerce_envia_shipping_settings', array() );
					$this->domain_url         = site_url();
					$this->domain_url         = site_url() == 'http://localhost/ecartTest' ? 'https://25f4-2806-109f-16-5949-3d02-c1ab-fe2c-b686.ngrok.io' : site_url();
					$this->active             = $this->get_option( 'active' );
					$this->useLabels          = $this->get_option( 'useLabels' );
					$this->inputFormCity      = $this->get_option( 'cityType' );
					// $this->availability = 'including';
					$this->init();
				}

				public function init() {
					$this->init_form_fields();
					$this->init_settings();

					add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
					if ( is_cart() || is_admin() || is_checkout() ) {
						if ( 'select' == $this->inputFormCity ) {
							wp_enqueue_script( 'formFieldsLoadAdmin', plugins_url( 'admin/js/formFieldsLoad.js', __FILE__ ), array(), '1.0', false );
							add_filter( 'woocommerce_default_address_fields', array( $this, 'custom_override_default_address_fields' ) );
						}
					}
					if ( is_admin() ) {
						wp_enqueue_style( 'enviaAdminstylesheet', plugins_url( 'admin/css/envia-shipping-admin.css', __FILE__ ), array(), '1.0', false );
					}
					if ( is_cart() || is_checkout() ) {
						if ( 'yes' == $this->useLabels ) {
							add_action( 'woocommerce_after_shipping_rate', 'action_after_shipping_rate', 20, 2 );
							wp_enqueue_style( 'enviaPublicStylesheet', plugins_url( 'public/css/envia-shipping-public.css', __FILE__ ), array(), '1.0', false );
						}
					}
				}

				public function init_form_fields() {
					$country = WC()->countries->get_base_country();
					$form    = array(
						// 'country'     => array(
						// 'title'   => '',
						// 'type'    => 'text',
						// 'default' => $country,
						// 'css'     => 'display:none',
						// 'class'   => 'select2-hidden-accessible',
						// ),
						'active'    => array(
							'title' => 'Active/Inactive',
							'type'  => 'checkbox',
							'label' => 'Enable or disable envia shipping method in your checkout.',
						),
						'useLabels' => array(
							'title' => 'Show label',
							'type'  => 'checkbox',
							'label' => 'Show image of carriers name in shipping rates options',
						),
						'token'     => array(
							'title' => 'API Key',
							'type'  => 'password',
							'class' => 'envia_shipping_disabled',
						),
						// 'originEmail' => array(
						// 'title' => 'Origin e-mail',
						// 'type'  => 'text',
						// ),
						'cityType'  => array(
							'title'       => 'Input type for city',
							'type'        => 'select',
							'label'       => 'Select type of input',
							'description' => __( 'Type of input in checkout/plugin page.', 'Tipo de boton en checkout/plugin page.' ),
							'options'     => array(
								'text'   => 'Text input',
								'select' => 'Dropdown list/Select',
							),
							'default'     => 'text',
						),
					);
					$form = array_merge( $form, $this->origin_address_form_fields() );
					// if ( $this->get_option( 'country' ) != '' && $this->get_option( 'country' ) != $country ) {
					// $pluginOptions  = get_option( 'woocommerce_envia_shipping_settings' );
					// $excludeOptions = array(
					// 'active',
					// 'token',
					// 'cityType',
					// 'useLabels',
					// 'enviaOrigin',
					// );
					// foreach ( $pluginOptions as $keyOpt => $option ) {
					// $auxOpt = false;
					// foreach ( $excludeOptions as $excludeOpt ) {
					// if ( $keyOpt == $excludeOpt ) {
					// $auxOpt = true;
					// break;
					// }
					// }
					// if ( ! $auxOpt ) {
					// $pluginOptions[ $keyOpt ] = '';
					// }
					// }
					// $pluginOptions['country'] = $country;
					// update_option( 'woocommerce_envia_shipping_settings', $pluginOptions );
					// }
					$this->form_fields                      = $form;
					$default                                = array( 'default' => 'Default origin address' );
					$formInstance                           = $this->origin_address_form_fields();
					$formInstance['enviaOrigin']['options'] = $formInstance['enviaOrigin']['options'] ? $default + $formInstance['enviaOrigin']['options'] : $default;
					$this->instance_form_fields             = $formInstance;
				}

				public function origin_address_form_fields() {
					try {
						$form                = array();
						$originOptions       = $this->get_envia_origin_adresses();
						$form['enviaOrigin'] = array(
							'title'       => 'Origin address',
							'type'        => 'select',
							'label'       => 'Envia origin address',
							'description' => 'It will be your default shipping origin address to use in the shipping calculator.',
							'options'     => $originOptions,
							'default'     => 'default',
						);

						if ( $this->get_instance_id() > 0 ) {
							$form['enviaOrigin']['description'] = 'It will be your default shipping origin address to use in the shipping calculator.<br><b>Default origin address:</b> Origin address selected in <a href="admin.php?page=wc-settings&tab=shipping&section=envia_shipping" target="_blank">plugin section</a> ';
							return $form;
						}
						return $form;
					} catch ( Exception $err ) {
						throw $err;
					}
				}

				public function get_envia_origin_adresses() {
					try {
						if ( false == $this->get_option( 'token' ) && '' == $this->get_option( 'token' ) ) {
							return null;
						}
						$addresses = $this->requests_process( 'GET', ENVIA_QUERIES_HOSTNAME . '/all-addresses/origin', $this->get_option( 'token' ), null );
						$origins   = array();
						foreach ( $addresses['data'] as $address ) {
							$origins[ $address['address_id'] ] = $address['street'] . ' #' . $address['number'] . ' ' . $address['state'] . '. ' . $address['country'] . '.';
						}
						return $origins;

					} catch ( Exception $e ) {
						throw $e;
					}
				}

				public function get_country_address_fields( $country = null ) {
					$country_form  = $this->requests_process( 'GET', ENVIA_QUERIES_HOSTNAME . '/generic-form?country_code=' . $country . '&form=address_info', null );
					$excludeFields = array(
						'district_select',
						'city_select',
					);
					if ( $this->get_option( 'cityType' ) == 'select' ) {
						$excludeFields[1] = 'city';
					}
					$class = 'envia_shipping_input';
					if ( $this->get_option( 'enviaOrigin' ) != 'default' && ! is_null( $this->get_option( 'enviaOrigin' ) ) ) {
						$class = $class . ' envia_shipping_disabled';
					}
					$formFields = array();
					foreach ( $country_form as $key => $value ) {
						$aux = false;
						foreach ( $excludeFields as $exclude ) {
							if ( $country_form[ $key ]['fieldId'] == $exclude ) {
								$aux = true;
								break;
							}
						}
						if ( ! $aux ) {
							if ( 'city_select' == $country_form[ $key ]['fieldId'] ) {
								$country_form[ $key ]['fieldId'] = 'city';
							}
							$formFields[ $country_form[ $key ]['fieldId'] ] = array(
								'title' => $country_form[ $key ]['fieldLabel'],
								'type'  => $country_form[ $key ]['fieldType'],
								'label' => $country_form[ $key ]['fieldLabel'],
								'class' => $class,
							);
						}
					}
					$formFields['state']['options'] = $this->get_country_states_options( $country );
					$currentCity                    = $this->get_option( 'city' );
					$formFields['city']['default']  = 'City';
					if ( $this->get_option( 'cityType' ) == 'select' ) {
						if ( '' != $currentCity ) {
							$formFields['city']['options'] = array( $currentCity => $currentCity );
						} else {
							$formFields['city']['options'] = array( 'default' => 'Select a city' );
						}
					}
					return $formFields;
				}

				public function get_country_states_options( $country ) {
					$states = WC()->countries->get_states( $country ) ? WC()->countries->get_states( $country ) : $this->requests_process( 'GET', ENVIA_QUERIES_HOSTNAME . '/state?country_code=' . $country, null );
					if ( isset( $states['data'] ) ) {
						$statesAux = array();
						foreach ( $states['data'] as $state ) {
							$statesAux[ $state['code_2_digits'] ] = $state['name'];
						}
						$states = $statesAux;
					}
					return $states;
				}

				public function admin_options() {
					?>
					<h2><?php echo esc_html( $this->method_title ); ?></h2>
					<p><?php echo esc_html( $this->method_description ); ?></p>
					<table class="form-table" style="margin-bottom: 10px;">
						<?php $this->generate_settings_html(); ?>
					</table>
					<p> Adds a new origin address from envia.com <a href="https://shipping.envia.com/settings/integrations" target="_blank" >here</a>.<br>*Use shipping zones if you need set a filter or a diferent origin address for zone.</p>
					<?php
				}

				public function calculate_shipping( $package = array() ) {
					if ( 'yes' == $this->active ) {
						try {
							global $woocommerce;
							$rates = $this->get_shipping_rates( $package );
							if ( isset( $rates['meta'] ) && 'error' == $rates['meta'] ) {
								throw new Exception( 'Envia shipping: ' . $rates['error']['message'], $rates['error']['code'] );
							}
							foreach ( $rates as $key => $value ) {
								$rate = array(
									'id'        => $key + 1,
									'label'     => $value['serviceDescription'],
									'cost'      => strval( $value['totalPrice'] ),
									'calc_tax'  => 'per_item',
									'meta_data' => array(
										'delivery' => $value['deliveryEstimate'],
										'carrier'  => $value['carrier'],
									),
									// 'package' => '$package'
								);
								$this->add_rate( $rate );
							}
						} catch ( Exception $err ) {
							// echo $err->getMessage();
							wc_add_notice( $err->getMessage(), 'notice' );
						}
					}
				}

				public function requests_process( $method, $url, $auth = null, $payload = null ) {
					if ( isset( $payload ) ) {
						$payload = json_encode( $payload );
					}
					try {
						$response = wp_remote_request(
							$url,
							array(
								'headers' => array(
									'Content-Type'  => 'application/json',
									'Authorization' => $auth,
								),
								'body'    => $payload,
								'method'  => $method,
							)
						);
						if ( is_wp_error( $response ) ) {
							$message = is_string( $response->get_error_code() ) ? $response->get_error_code() : $response->get_error_message();
							$code    = is_string( $response->get_error_code() ) ? 500 : $response->get_error_code();
							throw new Exception( $message, $code );
						} else {
							if ( $response['response']['code'] >= 400 ) {
								$response = json_decode( $response['body'], true );
								throw new Exception( $response['message'], $response['statusCode'] );
							} else {
								$data = json_decode( $response['body'], true );
								if ( array_key_exists( 'code', $data ) ) {
									$code = intval( $data['code'] );
									if ( $code >= 400 ) {
										throw new Exception( $response['message'], $response['statusCode'] );
									}
								}
								return json_decode( $response['body'], true );
							}
						}
					} catch ( Exception $err ) {
						throw $err;
					}
				}
				public function get_shipping_rates( $package ) {
					try {
						global $woocommerce;
						$url         = ENVIA_APP_HOSTNAME . '/v2/checkout/woocommerce/0?shop_url=' . $this->domain_url;
						$destination = $package['destination'];
						$cart        = $woocommerce->cart->get_cart();
						$origin      = $this->origin_options;
						// we can aggregate more fields like address, _1 _2 in destination
						$payload      = $this->create_format_request( $origin, $destination, $cart );
						$requestRates = $this->requests_process( 'POST', $url, null, $payload );
						return $requestRates;
					} catch ( Exception $err ) {
						throw $err;
					}
				}
				public function create_format_request( $origin, $destination, $cart ) {
					try {
						$dimensionsOf = $this->package_dimensions( $cart );
						$payload      = array(
							'origin'      => array(
								'address_id' => $this->get_instance_option( 'enviaOrigin' ) && $this->get_instance_option( 'enviaOrigin' ) != 'default' ? $this->get_instance_option( 'enviaOrigin' ) : $origin['enviaOrigin'],
							),
							'destination' => array(
								'name'       => 'envia',
								'company'    => 'envia',
								'email'      => 'clients@envia.com',
								'phone'      => '1234567890',
								'street'     => $destination['address'],
								'number'     => $destination['address_2'],
								'district'   => $destination['address_1'],
								'city'       => $destination['city'],
								'state'      => 'CO' == $destination['country'] ? explode( '-', $destination['state'] )[1] : $destination['state'],
								'country'    => $destination['country'],
								'postalCode' => $destination['postcode'],
							),
							'currency'    => get_woocommerce_currency(),
							'items'       => $dimensionsOf['items'],
							'packages'    => $dimensionsOf['packages'],
						);
						if ( 'default' == $payload['origin']['address_id'] ) {
							$payload['origin'] = array(
								'name'       => 'envia',
								'company'    => 'envia',
								'email'      => $origin['originEmail'] ? $origin['originEmail'] : 'clients@envia.com',
								'phone'      => '1234567890',
								'street'     => $origin['address1'],
								'number'     => $origin['address2'],
								'district'   => $origin['district'],
								'city'       => $origin['city'],
								'state'      => 'CO' == $origin['country'] ? explode( '-', $origin['state'] )[1] : $origin['state'],
								'country'    => $origin['country'],
								'postalCode' => $origin['postalCode'],
							);
						}
						return $payload;
					} catch ( Exception $err ) {
						throw $err;
					}
				}
				public function package_dimensions( $cart ) {
					try {
						$volumetricLimit = 68;
						$volumetric      = 0;
						$volumetricQuote = 0;
						$weight          = 0;
						$items           = array();
						$packages        = array();
						$lengthUnit      = get_option( 'woocommerce_dimension_unit' ) ? get_option( 'woocommerce_dimension_unit' ) : 'cm';
						$weightUnit      = get_option( 'woocommerce_weight_unit' ) ? get_option( 'woocommerce_weight_unit' ) : 'kg';
						foreach ( $cart as $element ) {
							$productId       = $element['product_id'];
							$variationId     = $element['variation_id'];
							$product         = wc_get_product( $productId );
							$height          = floatval( $product->get_height() );
							$length          = floatval( $product->get_width() );
							$width           = floatval( $product->get_length() );
							$variationWeight = 0;
							if ( $variationId > 0 ) {
								$variations = $product->get_available_variations();
								foreach ( $variations as $variation ) {
									if ( $variation['variation_id'] == $variationId ) {
										$height          = empty( floatval( $variation['dimensions']['height'] ) ) ? $height : floatval( $variation['dimensions']['height'] );
										$width           = empty( floatval( $variation['dimensions']['width'] ) ) ? $width : floatval( $variation['dimensions']['width'] );
										$length          = empty( floatval( $variation['dimensions']['length'] ) ) ? $length : floatval( $variation['dimensions']['length'] );
										$variationWeight = empty( floatval( $variation['weight'] ) ) ? 0 : floatval( $variation['weight'] );
									}
								}
							}
							$finalWeight = ( 0 != $variationWeight ? $variationWeight : ( ! empty( $product->get_weight() ) ? floatval( $product->get_weight() ) : 2 ) );
							for ( $i = 0; $i < $element['quantity']; $i++ ) {
								$volumetric     += ( $height * $width * $length ) / 5000;
								$weight         += $finalWeight;
								$volumetricQuote = $volumetric;
								$volumetricSub   = ( $height * $width * $length ) / 5000;
								if ( $volumetricQuote >= $volumetricLimit ) {
									$volumetricRate       = $volumetricQuote - $volumetricSub;
									$weight              -= $finalWeight;
									$volumetricDimensions = pow( ( $volumetricRate * 5000 ), 1 / 3 );
									$packages[]           = array(
										'content'       => 'Packages',
										'amount'        => 1,
										'type'          => 'box',
										'dimensions'    => array(
											'length' => $volumetricDimensions,
											'width'  => $volumetricDimensions,
											'height' => $volumetricDimensions,
										),
										'weight'        => $weight,
										'lengthUnit'    => $lengthUnit,
										'weightUnit'    => $weightUnit,
										'insurance'     => 0,
										'declaredValue' => 0,
									);
									$weight               = $finalWeight;
									$volumetric           = ( $height * $width * $length ) / 5000;
								}
							}

							$items[] = array(
								'name'               => $product->get_name(),
								'sku'                => $product->get_sku(),
								'quantity'           => $element['quantity'],
								'weight'             => $finalWeight,
								'price'              => $product->get_price(),
								'vendor'             => '',
								'requiresShipping'   => true,
								'taxable'            => true,
								'fulfillmentService' => 'manual',
								'properties'         => $height . ' * ' . $width . ' * ' . $length,
								'productId'          => $product->get_id(),
								'variantId'          => $variationId,
							);
						}
						$volumetricDimensions = pow( ( $volumetricQuote * 5000 ), 1 / 3 );
						$packages[]           = array(
							'content'       => 'Package',
							'amount'        => 1,
							'type'          => 'box',
							'dimensions'    => array(
								'length' => $volumetricDimensions,
								'width'  => $volumetricDimensions,
								'height' => $volumetricDimensions,
							),
							'weight'        => $weight,
							'lengthUnit'    => $lengthUnit,
							'weightUnit'    => $weightUnit,
							'insurance'     => 0,
							'declaredValue' => 0,
						);
						return array(
							'items'    => $items,
							'packages' => $packages,
						);
					} catch ( Exception $err ) {
							throw $err;
					}
				}
				public function custom_override_default_address_fields( $address_fields ) {
					$address_fields['city']['type']    = 'select';
					$address_fields['city']['options'] = array( 'default' => 'Select a city' );

					 return $address_fields;
				}
			}
		}
	}
}

add_action( 'plugins_loaded', 'initEnviaShipping' );

function add_envia_shipping_module( $methods ) {
	$methods['envia_shipping'] = 'Envia_Shipping_Module';
	return $methods;
}
add_filter( 'woocommerce_shipping_methods', 'add_envia_shipping_module' );

add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'add_plugin_page_settings_link' );
function add_plugin_page_settings_link( $links ) {
	$links[] = '<a href="' .
		admin_url( 'admin.php?page=wc-settings&tab=shipping&section=envia_shipping' ) .
		'">' . __( 'Settings' ) . '</a>';
	return $links;

}
		ob_end_flush();
